************************************************************************
*
*     Main Program: Mellin Representation for Heavy Quark Wilson Coefficients
*
*     [Extraction from the table 'minimax.tbl']
*
*     Sergey Alekhin and Johannes Bluemlein, January 2004
*
*     >>> CONDITION OF USE <<<
*     cite: 
*
*     S.I.Alekhin and J.Bluemlein, Phys.Lett. B594 (2004) 299.
*
*     Parts of the code are not subject to further distribution
*     without agreement of the authors of the code. 
*
************************************************************************
************************************************************************
*
*     Description of main aspects:
*     
*     The code can be compiled by:  g77 main.f
*     delivering a.out
*     COMMON files (see system of routines) have to be provided in the
*     same directory.
*     The execution of a.out requires reading from the table:
*     minimax.tbl 
*      
*     COMPILATION AND RUNNING:
*     ------------------------
*     Put main.f with the COMMON files ('includes') and minimax.tbl
*     into the same directory (or link accordingly).
*     g77 main.f leads to a.out which can be executed directly
*
*     OUTPUT:
*     ------
*     is written to lun=39 as fort.39.
*
*     ico,q2,xb,ffm    
*     ico=number of channel (1..14)
*     q2 = Q2 value    
*     xb = x  value
*     ffm= value of the function evaluated (>> ico)
*
*     The central function is:
*     real*8 function f2lhq(xi,xb,ico,qq,an)
*     (also applicable to g1 for ico=14).
*
*     TESTOUTPUT:
*     ----------
*     file: testoutput
*
*     In the files attached we incorporated certain very simple test-pdfs.
*     These have to be adapted to the user's problem for any realistic
*     analysis.
*     see: complex*16 function xg_mellin(ixi,icoi,cz)
*     for details.
*
************************************************************************
*
      PROGRAM HMELLIN
*     ---------------
*     version 1.00
*
      implicit real*8 (a-h, o-z)

      call inittables
      call initqgu

      ico=1                     ! F_2,g (LO), see comments in f2lhq  
*                                             for more options
      call testprint(ico)

      end
*
*=======================================================================
************************************************************************
*
*     J. BLUEMLEIN:  01.10.1999 (1.00)
*
*     BETA(A,B) FOR COMPLEX ARGUMENT
*
************************************************************************
*
      SUBROUTINE BETA(AA,BB,RES)
*     --------------------------

      COMPLEX*16 T1,T2,T3,T,RES,AA,BB
*
      CALL GAMMAL(AA,T1)
      CALL GAMMAL(BB,T2)
      CALL GAMMAL(AA+BB,T3)
      T=T1+T2-T3
*
      RES=EXP(T)
*
      RETURN
      END
*
*=======================================================================
************************************************************************
*
*     xi=q2/m^2
*     q2 - Q^2 variable
*     m  - mass of heavy quark
*     xb - Bjorken variable
*
*     ico=1 F_2,g (LO, gluon)
*     ico=2 F_L,g (LO, gluon)
*     ico=3 F_2,g (NLO, gluon)
*     ico=4 F_2,q (NLO, pure singlet)
*     ico=5 F_L,g (NLO, gluon)
*     ico=6 F_L,q (NLO, pure singlet)
*     ico=7 F_2,q (NLO, gluon radiation)
*     ico=8 F_L,q (NLO, gluon radiation)
*     ico=9 F_2,g (NLO, gluon, bar term)
*     ico=10 F_2,q (NLO, pure singlet, bar term)
*     ico=11 F_L,g (NLO, gluon, bar term)
*     ico=12 F_L,q (NLO, pure singlet, bar term)
*     ico=13 F_2,q (NLO, gluon radiation, bar term)
*     ico=14 G_1,g (LO, gluon)
*     
*     qq = charge of the heavy quark for gluon and pure singlet parts; 
*          1 for the gluon radiation parts*
*     
*     an - strong coupling constant
*
************************************************************************
*
      real*8 function f2lhq(xi,xb,ico,qq,an)
*     --------------------------------------
*
      implicit real*8 (a-h, o-z)
*
      include 'COEFF.inc'
*
      common /formellini/ xx,abs0,phi0
* 
c      external rmellini
*
      data  pi / 3.14159 26535 89793 23846 D+0 /
*      
      xx=xb
      phi0=pi*0.7d0
      abs0=0.3d0
*
      binxi=(log(ximax)-log(ximin))/(nxi-1)
      ixii=int((log(xi)-log(ximin))/binxi)+1
      del=(log(xi)-log(xigrid(ixii)))/binxi
c      CALL GAUSS1(ixii,xb,ico,ffm,EPS) 
      CALL GAUSS2(ixii,xb,ico,ffm,EPS) 
      ixii=ixii+1
c      CALL GAUSS1(ixii,xb,ico,ffp,EPS) 
      CALL GAUSS2(ixii,xb,ico,ffp,EPS) 
      ff=ffm+del*(ffp-ffm)
      ff=ff*xi*qq**2
*
      if (ico.le.2) ff=ff*an/4.d0/pi**2
      if (ico.gt.2.and.ico.le.13) ff=ff*an**2/pi
      if (ico.gt.13) ff=ff*an/pi
*
      f2lhq=ff
*
      return 
      end 
*
*=======================================================================
************************************************************************
*
*     J. BLUEMLEIN:  01.10.1999 (1.00)
*
*     LOG(GAMMA(Z)) FOR COMPLEX ARGUMENT
*
************************************************************************
*
      SUBROUTINE GAMMAL(ZZ,RES)
*     -------------------------

      implicit real*8 (a-h, o-z)
*
      COMPLEX*16 T1,T2,ZZ,Z,X,X2,RES,t
*
      Z=ZZ
      PI = 3.14159 26535 89793 23846 2643 D+0
*
      ONE=DCMPLX(1.0D0,0.0D0)
      T=DCMPLX(0.0D0,0.0D0)
2     R=SQRT(DREAL(Z)**2+DIMAG(Z)**2)
      IF(R.GT.10.0D0) GOTO 1
      T=T-LOG(Z)
      Z=Z+ONE
      GOTO 2
1     CONTINUE
*
      T1=Z*(LOG(Z)-1.0D0)+LOG(2.0D0*PI/Z)/2.0D0
*
      X=ONE/Z
      X2=X*X
      T2 = (1.D0/12.D0+(-1.D0/360.D0+(1.D0/1260.D0+(-1.D0/1680.D0+(1.D0/
     #1188.D0+(-691.D0/360360.D0+(1.D0/156.D0-3617.D0/122400.D0*X2)*X2
     #)*X2)*X2)*X2)*X2)*X2)*X
*
      RES=T1+T2+T
*
      RETURN
      END
*
*=======================================================================
************************************************************************
*
*     12 point GAUSS quadrature / Sergey Alekhin
*
************************************************************************
*
      SUBROUTINE GAUSS1(ixi,xx,ico,RESULT,EPS)
*     ----------------------------------------
*
      implicit real*8 (a-h, o-z)
*
      parameter (kint=15)
*
      include 'COEFF.inc'
*
      complex*16 coeffbuf(nxi,ncoef,kint,-12:12)
      complex*16 cz,dir,trans,xg_mellin
*
      save init,coeffbuf,phi0,abs0,a,b,c0,dir,c
*
      DIMENSION W(12),X(12)
*
      DATA W
     1/ 0.10122 85362 90376,  0.22238 10344 53374,  0.31370 66458 77887,
     2  0.36268 37833 78362,  0.02715 24594 11754,  0.06225 35239 38648,
     3  0.09515 85116 82493,  0.12462 89712 55534,  0.14959 59888 16577,
     4  0.16915 65193 95003,  0.18260 34150 44924,  0.18945 06104 55069/
      DATA X
     1/ 0.96028 98564 97536,  0.79666 64774 13627,  0.52553 24099 16329,
     2  0.18343 46424 95650,  0.98940 09349 91650,  0.94457 50230 73233,
     3  0.86563 12023 87832,  0.75540 44083 55003,  0.61787 62444 02644,
     4  0.45801 67776 57227,  0.28160 35507 79259,  0.09501 25098 37637/
*
      data  PI / 3.14159 26535 89793 23846 D+0 /
      data init /0/
*
      if (init.eq.0) then
         phi0=pi*0.7
c         phi0=pi*0.75
         c0=0.3 
         dir=dcmplx(cos(phi0),sin(phi0))
c         a=-20.
         a=0.0
c         b=6.0
         b=400.0
         C=(B-A)/kint
         do i1=1,nxi
            do i2=1,ncoef
               au=a
               do i3=1,kint
                  FI=i3
                  AO=A+FI*C
                  C1=0.5*(AU+AO)
                  C2=C1-AU
                  do i4=1,12
                     U=X(i4)*C2
c                     zn=exp(c1+u)
                     zn=(c1+u)
                     cz=c0+zn*dir
                     call mellin (i2,i1,cz,trans)
                     coeffbuf(i1,i2,i3,i4)=trans
c                     zn=exp(c1-u)
                     zn=(c1-u)
                     cz=c0+zn*dir
                     call mellin (i2,i1,cz,trans)
                     coeffbuf(i1,i2,i3,-i4)=trans
                  end do
                  au=ao
               end do
            end do
         end do
         init=1
      end if
*
      au=a
      RESULT=0.
      EPS=0.
*
      do I = 1,kint
         FI=I
         AO=A+FI*C
         C1=0.5*(AU+AO)
         C2=C1-AU
         S8=0.
         S16=0.
*
         do J = 1,4
            U=X(J)*C2
c            zn=exp(c1-u)
            zn=(c1-u)
            cz=c0+zn*dir
            trans=coeffbuf(ixi,ico,i,-j)
            trans=trans*dir*(dcmplx(xx,0d0))**(-cz)
            trans=trans*xg_mellin(ixi,ico,cz)
c            rmm=dimag(trans)/pi*zn
            rmm=dimag(trans)/pi
c            zn=exp(c1+u)
            zn=(c1+u)
            cz=c0+zn*dir
            trans=coeffbuf(ixi,ico,i,j)
            trans=trans*dir*(dcmplx(xx,0d0))**(-cz)
            trans=trans*xg_mellin(ixi,ico,cz)
c            rmp=dimag(trans)/pi*zn
            rmp=dimag(trans)/pi
            S8=S8+W(J)*(rmp+rmm)
         end do
*
         do J = 5,12
            U=X(J)*C2
c            zn=exp(c1-u)
            zn=(c1-u)
            cz=c0+zn*dir
            trans=coeffbuf(ixi,ico,i,-j) 
            trans=trans*dir*(dcmplx(xx,0d0))**(-cz)
            trans=trans*xg_mellin(ixi,ico,cz)
c            rmm=dimag(trans)/pi*zn
            rmm=dimag(trans)/pi
c            zn=exp(c1+u)
            zn=(c1+u)
            cz=c0+zn*dir
            trans=coeffbuf(ixi,ico,i,j)
            trans=trans*dir*(dcmplx(xx,0d0))**(-cz)
            trans=trans*xg_mellin(ixi,ico,cz)
c            rmp=dimag(trans)/pi*zn
            rmp=dimag(trans)/pi
            S16=S16+W(J)*(rmp+rmm)
         end do
*
         RESULT=RESULT+C2*S16
         EPS=EPS+ABS(C2*(S16-S8))
         AU=AO
*
      end do
*
      return
      end
*
*=======================================================================
      subroutine gauss2(ixi,xb,ico,fff,EPS)
*     -------------------------------------
*
      implicit none
*
      INTEGER NI, NP, ND
      PARAMETER (NI = 17, NP = 8, ND = NI*NP)
*
      integer ixi, ico
      integer IC, KN
*
      real*8 xb, fff, EPS
      real*8 PA
*
      COMPLEX*16 XN, WCHQ, xg_mellin
*
      include 'COEFF.inc'
*
      REAL*8 WN
      COMMON / WEIGHTS / WN(ND)
      REAL*8 C
      COMPLEX*16 CC
      COMMON / NCONT / C, CC
      COMPLEX*16 CN
      COMMON / MOMS  / CN(ND)
      COMPLEX*16 FN
      COMMON / FNMOM / FN(ND)
*
*-----------------------------------------------------------------------
*
      do KN = 1,ND
         XN = CN(KN)
         call mellin(ico,ixi,XN,WCHQ)
         FN(KN) = WCHQ * xg_mellin(ixi,ico,XN)
      enddo
*
      call invunp (PA, xb)
*
      fff = PA
      EPS = 0.0d0
*
      return
      end
*
*=======================================================================
************************************************************************
*
*     File: initqgu.f    (calls to ANOMQGU and COMBQG)
*
*     Initialization of support points in N-space and weights for the
*     Gauss quadrature; and of the anomalous dimensions etc for the
*     partonic RG evolution at these N-values.
*
*     The outputs are written to the common-blocks NCONT, MOMS, WEIGHTS.
*     The integration is divided into NI = 17 intervals with an 8-point 
*     Gauss quadrature performed on each of them.
*
************************************************************************
*
      SUBROUTINE INITQGU
*     ------------------
*
      IMPLICIT NONE
*
      INTEGER NI, NP, ND
      PARAMETER (NI = 17, NP = 8, ND = NI*NP)
*
      INTEGER KN, II, IP, IC
*
      REAL*8 ZS(NP), WZ(NP), DOWN(NI), UP(NI) 
      REAL*8 PI, PHI, SUM, DIFF, Z
*
      COMPLEX*16 CI 
*
*
* ---------------------------------------------------------------------
*
*     OUTPUT COMMON BLOCKS :
*
      REAL*8 WN
      COMMON / WEIGHTS / WN(ND)
*
      REAL*8 C
      COMPLEX*16 CC
      COMMON / NCONT / C, CC
      COMPLEX*16 CN
      COMMON / MOMS  / CN(ND)
*
* ---------------------------------------------------------------------
*
*     WEIGHTS AND SUPPORT POINTS FOR NORMALIZED 8 POINT GAUSS QUADRATURE:
*
      DATA WZ
     1     / 0.10122 85362 90376 D0,  0.22238 10344 53374 D0, 
     2       0.31370 66458 77887 D0,  0.36268 37833 78362 D0, 
     3       0.36268 37833 78362 D0,  0.31370 66458 77887 D0,
     4       0.22238 10344 53374 D0,  0.10122 85362 90376 D0/
      DATA ZS
     1     /-0.96028 98564 97536 D0, -0.79666 64774 13627 D0, 
     2      -0.52553 24099 16329 D0, -0.18343 46424 95650 D0, 
     3       0.18343 46424 95650 D0,  0.52553 24099 16329 D0,
     4       0.79666 64774 13627 D0,  0.96028 98564 97536 D0/
*
* ---------------------------------------------------------------------
* 
*     INTEGRATION PARAMETERS :
*     (LOWER AND UPPER LIMITS FOR THE INTERVALS, LOCATION OF THE CONTOUR
*     IN THE COMPLEX N PLANE) :
*
      DATA DOWN / 0.0D0, 0.1D0, 0.3D0, 0.6D0, 1.0D0, 1.6D0, 2.4D0,
     1            3.5D0, 5.0D0, 7.0D0,10.0D0,14.0D0,19.0D0,25.0D0,
     2           32.0D0,40.0D0,52.0D0 /
*
      DATA PI / 3.14159 26535 89793 23846 d0 /
*
      C = 1.0D0 
c      C = 0.3D0 
      CI = DCMPLX (0.0D0, 1.0D0)
      PHI = PI * 0.75d0
      CC  = EXP (CI*PHI)
      DO II = 1, NI-1
         UP(II) = DOWN(II+1)
      ENDDO
      UP(NI) = 65.0D0
* 
* ---------------------------------------------------------------------
*
*     SUPPORT POINTS CN(K) AND WEIGHTS WN(K) FOR THE GAUSS INTEGRATIONS :
*     (THE FACTOR [UPPER LIMIT - LOWER LIMIT]/2 IS PUT INTO THE WEIGHTS)
*
      KN = 0
      DO II = 1, NI
         SUM  = UP(II) + DOWN(II) 
         DIFF = UP(II) - DOWN(II) 
         DO IP = 1, NP
            KN = KN + 1
            Z = 0.5D0 * (SUM + DIFF * ZS(IP))
            WN(KN) = DIFF * 0.5D0 * WZ(IP) 
            CN(KN) = CC * Z + C + 1.0D0 
         ENDDO
      ENDDO
*
* ---------------------------------------------------------------------
*
      RETURN
      END 
*
* =====================================================================
************************************************************************
*
*     read in minimax table
*
************************************************************************
*
      subroutine inittables
*     ---------------------
*
      implicit real*8 (a-h, o-z)
*
      include 'COEFF.inc'
*
      open (unit=41,file='minimax.tbl',status='OLD')
      read (41,fmt='(10i10)') m_coef
      read (41,fmt='(6e23.16)') ximin,ximax,xigrid,
     1    coef_p1,coef_p2,coef
*
      return 
      end
*
*=======================================================================
************************************************************************
*
*     File: invunp.f
*
*     Unpolarized parton densites and/or structure functions for light 
*     hadrons (no s-quark or heavier valence component) at externally 
*     fixed Bjorken-x, XB, and scale Q**2 in GeV**2, QS, calculated via 
*     a Mellin inversion of the analytic RGE solution determined in 
*     RENOQGU and EVOL_LO/EVOL_NLO.
*
*     The output is written to PA; 
*     for parton densities always x times the distribution is returned. 
*
************************************************************************
*
      SUBROUTINE INVUNP (PA, X)
*     -------------------------
*
      IMPLICIT none
*
      INTEGER ND
      PARAMETER (ND = 136)
*
      INTEGER NMAX, I1, I3
*
      REAL*8 PA, X 
      REAL*8 PI, AX, EX, FUN, FZ
*
      COMPLEX*16 C1, CCP, CXNM, CEX(ND)
*
* ---------------------------------------------------------------------
*
*     COMMON BLOCKS :
*
      REAL*8 C
      COMPLEX*16 CC                              
      COMMON / NCONT   / C, CC                              
      REAL*8 WN
      COMMON / WEIGHTS / WN(ND)
      COMPLEX*16 CN
      COMMON / MOMS    / CN(ND)
      COMPLEX*16 FN
      COMMON / FNMOM / FN(ND)
*
      DATA PI / 3.14159 26535 89793 23846 d0 /
*
* ---------------------------------------------------------------------
*
*     Q2- AND X-DEPENDENT QUANTITIES : 
*
      AX = LOG (X)
      EX = EXP (-C * AX)
*     
* ----------------------------------------------------------------------
*     
*     INTEGRATION LENGTH PARAMETER FOR THE MELLIN INVERSION :     
*
      IF ( X .LT. 0.01D0 ) THEN                          
         NMAX = 64          
*        (zmax = 5) 
      ELSE IF ( X .LT. 0.3D0 ) THEN                   
         NMAX = 88 
*        (zmax = 10) 
      ELSE IF ( X .LT. 0.7D0 ) THEN                   
         NMAX = 112  
*        (zmax = 25) 
      ELSE                                                          
         NMAX = 136   
*        (zmax = 65) 
      END IF
*     
* ----------------------------------------------------------------------
*
*     CALCULATION OF THE PARTON DENSITIES, STRUCTURE FCT'S, AND OUTPUT :
*
*     
*     CONTOUR DEPENDENCE IN ACTION
*
      C1  = C + 1.0D0
      CCP = CC / PI
      DO I3 = 1, NMAX
         CXNM = (C1 - CN(I3)) * AX
         CEX(I3) = EXP (CXNM) * CCP
      ENDDO
*
*     BEGIN INVERSION LOOP :
*
      FUN = 0.0D0
      DO I1 = 1, NMAX
         FZ  = DIMAG (FN(I1)*CEX(I1))
         FUN = FUN + WN(I1) * FZ
      ENDDO
*
      PA = FUN * EX / X
*
      RETURN                                                           
      END                                                             
*
* =====================================================================
************************************************************************
*
      subroutine mellin (icoef,ixi,cz,trans)
*     --------------------------------------
*
      implicit real*8 (a-h, o-z)
*
      include 'COEFF.inc'
*
      complex*16 cz,trans,a,b,res
*
      xth=1.d0/(4.d0/xigrid(ixi)+1.d0)
*
      trans=0.d0
*
      do i=1,m_coef(icoef)
        a=i-1.d0+cz-coef_p1(icoef)
        b=1.d0-coef_p2(icoef)
        call beta(a,b,res)
        trans=trans+coef(icoef,ixi,i)*xth**(a+b-1.d0)*res
      end do
*
      return 
      end
*
*=======================================================================
************************************************************************
*
      real*8 function rmellini (ixii,icoi,t)
*     --------------------------------------
*
      implicit real*8 (a-h, o-z)
*
      common /formellini/ xx,abs0,phi0
*
      complex*16 trans,dir,cz,res,xg_mellin
*
      data  pi / 3.14159265358979323846D+0 /                                  
*    
      c0=abs0
      zn=exp(t)
      dir=dcmplx(cos(phi0),sin(phi0))
      cz=c0+zn*dir
      call mellin(icoi,ixii,cz,trans)
      trans=trans*dir*(dcmplx(xx,0d0))**(-cz)
      trans=trans*xg_mellin(ixii,icoi,cz)
*
      rmellini=dimag(trans)/pi*zn
*
      return 
      end
*
*=======================================================================
************************************************************************
*
*     test program: calculates nx values
*
************************************************************************
*
      subroutine testprint(ico)
*     -------------------------
*
      implicit real*8 (a-h, o-z)
*
      DATA  pi / 3.14159 26535 89793 23846 D+0 /
*
      rm2=1.5d0**2
      an=0.2d0
      sher=1.d5
      qqs=2.d0/3.d0
      nx=100
*      
c      do iq=1,5
      do iq=3,3
        q2=10.d0**(iq-1)
        xi=q2/rm2
        xth=1.d0/(4.d0/xi+1.d0)
        xbmin=xi*rm2/sher
        xbmax=xth
        binx=(log(xbmax)-log(xbmin))/(nx-1)                      
        print *,'*** calculation for Q2 = ',q2,' GeV^2',
     1                             ' xi = ',xi,' ***'
        do i=1,nx-1
          xb=exp(log(xbmin)+binx*(i-1))
          ffm=f2lhq(xi,xb,ico,qqs,an)
          write(39,*) ico,q2,xb,ffm
        end do
      end do
* 
      return 
      end
*
*=======================================================================
************************************************************************
*
*     parameterization in Mellin-N space
*
*     see Ref.: S.I.Alekhin and J.Bluemlein, Phys.Lett.B594(2004)299
*
************************************************************************
*
      complex*16 function xg_mellin(ixi,icoi,cz)
*     ------------------------------------------
*
      implicit real*8 (a-h, o-z)
*
      common /formellini/ xx,abs0,phi0
*
      complex*16 cz,res
*
*     gluon 
*
      if (icoi.le.3.or.icoi.eq.5.or.icoi.eq.9.or.icoi.eq.11) then 
         a=-0.2d0
         b=5.d0
         gnor=1.8d0
      end if
*
*     u + ubar + d + dbar + s + sbar 
*
      if (icoi.eq.4.or.icoi.eq.6.or.icoi.eq.10.or.icoi.eq.12) then 
         a=-0.2d0
         b=5.d0
         gnor=0.6d0
      end if
*
*     4/9*(u+ubar) + 2/9*(d+dbar) + 1/9*(s+sbar)
*
      if (icoi.eq.7.or.icoi.eq.8.or.icoi.eq.13) then 
        a=-0.2d0
        b=5.d0
        gnor=0.15d0
      end if
*
*     gluon polarization 
*
      if (icoi.eq.14) then 
        a=1.4d0
        b=6.d0
        gnor=20.d0
      end if
*
      call beta(cz+a,dcmplx(b+1.d0,0.d0),res)
      xg_mellin=gnor*res
*
      return 
      end
*
*=======================================================================
